import {
  Injectable,
  InternalServerErrorException,
  Logger,
  NotFoundException,
  UnauthorizedException,
} from '@nestjs/common'
import { JwtService } from '@nestjs/jwt'
import { InjectModel } from '@nestjs/mongoose'
import { Model } from 'mongoose'
import { User } from 'src/schemas/user.schema'
import bcrypt from 'bcrypt'
import { RegisterDto } from './dto/register.dto'
import { LoginDto } from './dto/login.dto'
import { randomUUID } from 'crypto'
import { ConfirmDto } from './dto/confirm.dto'
import { UpdatePasswordDto } from './dto/update-password.dto'

@Injectable()
export class AuthService {
  private readonly logger = new Logger(AuthService.name);

  constructor(
    @InjectModel(User.name) private userModel: Model<User>,
    private jwtService: JwtService,
  ) {}

  async login(body: LoginDto): Promise<{ token: string }> {
    const user = await this.userModel.findOne({ email: body.email })

    if (!user) throw new UnauthorizedException('wrong credentials')

    if (!(await bcrypt.compare(body.password, user.password)))
      throw new UnauthorizedException('wrong credentials')

    return { token: this.jwtService.sign(user.email) }
  }

  async register(body: RegisterDto) {
    try {
      const hashedPassword = await bcrypt.hash(body.password, 10)

      await this.userModel.create({
        ...body,
        password: hashedPassword,
        registrationToken: randomUUID(),
      })
    } catch (err) {
      this.logger.error('Could not register user', err)
      throw new InternalServerErrorException()
    }
  }

  async confirmRegistration(body: ConfirmDto) {
    const user = await this.userModel.findById(body.id)

    if (!user) throw new NotFoundException('user not found')

    if (user.registrationToken != body.token)
      throw new UnauthorizedException('wrong registration token')

    user.status = 'CONFIRMED'
    await user.save()
  }

  async updatePassword(body: UpdatePasswordDto) {
    const user = await this.userModel.findById(body.id)

    if (!user) throw new NotFoundException('user not found')

    if (!(await bcrypt.compare(body.oldPassword, user.password)))
      throw new UnauthorizedException('wrong credentials')

    user.password = await bcrypt.hash(body.newPassword, 10)
    await user.save()
  }
}
