import {
  Injectable,
  NotFoundException,
  UnauthorizedException,
} from '@nestjs/common'
import { CreateConnectionDto } from './dto/create-connection.dto'
import { UpdateConnectionDto } from './dto/update-connection.dto'
import { InjectModel } from '@nestjs/mongoose'
import { Model } from 'mongoose'
import { JwtService } from '@nestjs/jwt'
import { User } from 'src/schemas/user.schema'
import { Connection } from 'src/schemas/connection.schema'

@Injectable()
export class ConnectionsService {
  constructor(
    @InjectModel(Connection.name) private connectionModel: Model<Connection>,
    @InjectModel(User.name) private userModel: Model<User>,
    private jwtService: JwtService,
  ) {}

  // creates a pending request.
  async create(authHeader: string, createConnectionDto: CreateConnectionDto) {
    const decodedJwt = this.jwtService.decode(authHeader.split(' ')[1])
    const userId = decodedJwt.userId

    if (!userId) throw new UnauthorizedException()

    const reciever = await this.userModel.findOne({
      _id: createConnectionDto.reciever,
    })

    if (!reciever) throw new NotFoundException()

    return await this.connectionModel.create({
      sender: userId,
      reciever: reciever.id,
      status: 'PENDING',
    })
  }

  // returns pending connections for the requesting user.
  async pending(authHeader: string) {
    const decodedJwt = this.jwtService.decode(authHeader.split(' ')[1])
    const userId = decodedJwt.userId

    if (!userId) throw new UnauthorizedException()

    return await this.connectionModel.find({
      reciever: userId,
      status: 'PENDING',
    })
  }

  // accepts a recieved connection. returns unauthorized if the requesting user is
  // not the reciever.
  async accept(authHeader: string, _id: number) {
    const decodedJwt = this.jwtService.decode(authHeader.split(' ')[1])
    const userId = decodedJwt.userId

    if (!userId) throw new UnauthorizedException()

    const connection = await this.connectionModel.findOne({ _id })

    if (!connection) throw new NotFoundException()

    if (connection.reciever != userId) throw new UnauthorizedException()

    return await this.connectionModel.findOneAndUpdate(
      { _id },
      { status: 'ACTIVE' },
    )
  }

  // declines a recieved connection. returns unauthorized if the requesting user is
  // not the reciever.
  async decline(authHeader: string, _id: number) {
    const decodedJwt = this.jwtService.decode(authHeader.split(' ')[1])
    const userId = decodedJwt.userId

    if (!userId) throw new UnauthorizedException()

    const connection = await this.connectionModel.findOne({ _id })

    if (!connection) throw new NotFoundException()

    if (connection.reciever != userId) throw new UnauthorizedException()

    return await this.connectionModel.findOneAndDelete({ _id })
  }

  // deletes a sent connection. returns unauthorized if the requesting user is
  // not the sender.
  async remove(authHeader: string, _id: number) {
    const decodedJwt = this.jwtService.decode(authHeader.split(' ')[1])
    const userId = decodedJwt.userId

    if (!userId) throw new UnauthorizedException()

    const connection = await this.connectionModel.findOne({ _id })

    if (!connection) throw new NotFoundException()

    if (connection.sender != userId) throw new UnauthorizedException()

    return await this.connectionModel.findOneAndDelete({ _id })
  }
}
